/****************************************************************************/
/* Copyright (C) 2009-2011, Heartland.Data.co.  All Rights Reserved.		*/
/*																			*/
/* Tittle	: TP_Busout														*/
/* Author	: HLDC															*/
/****************************************************************************/

#include <stdio.h>								// Header file for Standard I/O.
#include "DTdrv.h"
	
/* ========== Parameter ========== */
/* Please customize the code for your environment. */ 
#define		DT_UINT32			unsigned int			/* Test Point argument type */
/* =============================== */

/* ========== Don't Change Parameter ========== */
/* Please do not change the value. */ 
#define		DT_VARIABLE_BIT			0x02
#define		DT_EVTTRG_BIT			0x08
/* ============================================ */

static	FILE *dat_file = NULL ;
static	unsigned long time_count = 0;
static	int		BusoutMode = 0;			/* BusOutMode setting. The default is 0. */


/*===============================================================================
	Func	: BusOutModeSet
	Desc	: When you want to change the BusOutMode, please use it in any.
    Mode	: 0 -> Create the file.
              1 -> The acquisition of the Test Report.
              2 -> The acquisition of the Test Report stops.
===============================================================================*/
void BusOutModeSet( int mode )			
{
	BusoutMode = mode ;
}


/*===============================================================================
	Func	: _TP_BusOut
	Desc	: Called by Test Point
===============================================================================*/
void _TP_BusOut( DT_UINT32 addr, DT_UINT32 dat )
{
	char buff[16] = {0};

	if( BusoutMode == 0 )
	{
		dat_file = fopen("/mnt/sdcard/Convertdata.DAT","wb+"); 		// [Convertdata.DAT] file is opened. File name is arbitrary.
		BusoutMode = 1 ;
		time_count = 0;
	}

	buff[0] = (unsigned char)((dat >> 0) & 0xFF);	//dat
	buff[1] = (unsigned char)((dat >> 8) & 0xFF);	//dat

	buff[2] = (unsigned char)((addr >> 0) & 0xFF);	//addr
	buff[3] = (unsigned char)((addr >> 8) & 0xFF);	//addr
	buff[4] = (unsigned char)((addr >> 16) & 0xFF);	//addr

	time_count = time_count + 1;		// Please describe the code to set the time info when you add Time Value. 

	buff[5]  = (unsigned char)((time_count >> 0) & 0xFF);	//Please enter the time info.
	buff[6]	 = (unsigned char)((time_count >> 8) & 0xFF);
	buff[7]  = (unsigned char)((time_count >> 16) & 0xFF);
	buff[8]  = (unsigned char)((time_count >> 24) & 0xFF);

	buff[9] = 0x00;
	buff[10] = 0x00;	
	buff[11] = 0x00;
	buff[12] = 0x00;
	buff[13] = 0x00;
	buff[14] = 0x00;
	buff[15] = 0x00;

	if( dat_file != NULL )
	{
		fwrite(buff,sizeof(char),16,dat_file);
	}

	if( BusoutMode == 2 )
	{
		fclose(dat_file);
	}

}

/*===============================================================================
	Func	: _TP_MemoryOutput
	Desc	: Called by Variable Test Point
===============================================================================*/
void _TP_MemoryOutput( DT_UINT32 addr, DT_UINT32 dat, void *value, DT_UINT32 size)
{
	char	buff[16];
	char	*p = value;
	int		i, j, packetNum, count;
	
	if( size >= 256 ) size = 256;
	packetNum = (size + 7) / 8;
	
	_TP_BusOut( addr|DT_VARIABLE_BIT, dat );		/* Memory Dump Start */ 
	
	buff[ 0] = size;
	buff[ 1] = packetNum;
	buff[ 2] = 0x03;
	buff[12] = 0x00;
	buff[13] = 0x00;
	buff[14] = 0x00;
	buff[15] = 0x00;
	
	count = 0;
	for( i = 0; i < packetNum; i++ ){
		buff[3] = i;
		for( j = 0; j < 8; j++, count++, p++ ){
			buff[4+j] = (count < size)?*p:0x00;
		}
		if( dat_file != NULL )
		{
			fwrite(buff,sizeof(char),16,dat_file);
		}
	}

}

/*===============================================================================
	Func	: _TP_MemoryOutput
	Desc	: Called by Variable Test Point
===============================================================================*/
void _TP_EventTrigger( DT_UINT32 addr, DT_UINT32 dat)
{
	_TP_BusOut( addr | DT_EVTTRG_BIT, ((unsigned short)(dat&0x0FFF))<<4 );
}

//------------------------------------------------------------------------------
//	JNI Function
//------------------------------------------------------------------------------
void Java_com_heartland_block_DTdrv_OpenFile(JNIEnv *env, jobject obj)
{
	if( BusoutMode == 0 )
	{
		dat_file = fopen("/mnt/sdcard/Convertdata.DAT","wb+"); 		/* [Convertdata.DAT]t@C̃I[vBt@C̓[U[̔CӂłB */
		BusoutMode = 1 ;
		time_count = 0;
	}
}

/*
 * Class:     com_heartland_block_DTdrv
 * Method:    CloseFile
 * Signature: ()V
 */
void Java_com_heartland_block_DTdrv_CloseFile(JNIEnv *env, jobject obj)
{
	fclose(dat_file);
}

void Java_com_heartland_block_DTdrv_TPBusOut(JNIEnv *env, jobject obj, jint nAddress, jint nData)
{
	_TP_BusOut(nAddress, nData);
}

void Java_com_heartland_block_DTdrv_TPMemoryOutput__IIBI(JNIEnv *env, jobject obj, jint nAddress, jint nData, jbyte theValue, jint size)
{
	_TP_MemoryOutput(nAddress, nData, (void *)&theValue, size);
}

void Java_com_heartland_block_DTdrv_TPMemoryOutput__IICI(JNIEnv *env, jobject obj, jint nAddress, jint nData, jchar theValue, jint size)
{
	_TP_MemoryOutput(nAddress, nData, (void *)&theValue, size);
}

void Java_com_heartland_block_DTdrv_TPMemoryOutput__IISI(JNIEnv *env, jobject obj, jint nAddress, jint nData, jshort theValue, jint size)
{
	_TP_MemoryOutput(nAddress, nData, (void *)&theValue, size);
}

void Java_com_heartland_block_DTdrv_TPMemoryOutput__IIII(JNIEnv *env, jobject obj, jint nAddress, jint nData, jint theValue, jint size)
{
	_TP_MemoryOutput(nAddress, nData, (void *)&theValue, size);
}

void Java_com_heartland_block_DTdrv_TPMemoryOutput__IIJI(JNIEnv *env, jobject obj, jint nAddress, jint nData, jlong theValue, jint size)
{
	_TP_MemoryOutput(nAddress, nData, (void *)&theValue, size);
}

void Java_com_heartland_block_DTdrv_TPMemoryOutput__IIZI(JNIEnv *env, jobject obj, jint nAddress, jint nData, jboolean theValue, jint size)
{
	_TP_MemoryOutput(nAddress, nData, (void *)&theValue, size);
}

void Java_com_heartland_block_DTdrv_TPMemoryOutput__II_3BII(JNIEnv *env, jobject obj, jint nAddress, jint nData, jbyteArray theValue, jint size, jint nLength)
{
	_TP_MemoryOutput(nAddress, nData, (void *)&theValue, size*nLength);
}

void Java_com_heartland_block_DTdrv_TPMemoryOutput__II_3CII(JNIEnv *env, jobject obj, jint nAddress, jint nData, jcharArray theValue, jint size, jint nLength)
{
	_TP_MemoryOutput(nAddress, nData, (void *)&theValue, size*nLength);
}

void Java_com_heartland_block_DTdrv_TPMemoryOutput__II_3SII(JNIEnv *env, jobject obj, jint nAddress, jint nData, jshortArray theValue, jint size, jint nLength)
{
	_TP_MemoryOutput(nAddress, nData, (void *)&theValue, size*nLength);
}

void Java_com_heartland_block_DTdrv_TPMemoryOutput__II_3III(JNIEnv *env, jobject obj, jint nAddress, jint nData, jintArray theValue, jint size, jint nLength)
{
	_TP_MemoryOutput(nAddress, nData, (void *)&theValue, size*nLength);
}

void Java_com_heartland_block_DTdrv_TPMemoryOutput__II_3JII(JNIEnv *env, jobject obj, jint nAddress, jint nData, jlongArray theValue, jint size, jint nLength)
{
	_TP_MemoryOutput(nAddress, nData, (void *)&theValue, size*nLength);
}

void Java_com_heartland_block_DTdrv_TPMemoryOutput__II_3ZII(JNIEnv *env, jobject obj, jint nAddress, jint nData, jbooleanArray theValue, jint size, jint nLength)
{
	_TP_MemoryOutput(nAddress, nData, (void *)&theValue, size*nLength);
}
